# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.compute import ComputeManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-compute
# USAGE
    python restore_point_collection_update_minimum_set_gen.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ComputeManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.restore_point_collections.update(
        resource_group_name="rgcompute",
        restore_point_collection_name="aaaaaaaaaaaaaaaaaa",
        parameters={
            "properties": {
                "instantAccess": bool,
                "provisioningState": "str",
                "restorePointCollectionId": "str",
                "restorePoints": [
                    {
                        "id": "str",
                        "name": "str",
                        "properties": {
                            "consistencyMode": "str",
                            "excludeDisks": [{"id": "str"}],
                            "instanceView": {
                                "diskRestorePoints": [
                                    {
                                        "id": "str",
                                        "replicationStatus": {
                                            "completionPercent": 0,
                                            "status": {
                                                "code": "str",
                                                "displayStatus": "str",
                                                "level": "str",
                                                "message": "str",
                                                "time": "2020-02-20 00:00:00",
                                            },
                                        },
                                        "snapshotAccessState": "str",
                                    }
                                ],
                                "statuses": [
                                    {
                                        "code": "str",
                                        "displayStatus": "str",
                                        "level": "str",
                                        "message": "str",
                                        "time": "2020-02-20 00:00:00",
                                    }
                                ],
                            },
                            "instantAccessDurationMinutes": 0,
                            "provisioningState": "str",
                            "sourceMetadata": {
                                "diagnosticsProfile": {"bootDiagnostics": {"enabled": bool, "storageUri": "str"}},
                                "hardwareProfile": {
                                    "vmSize": "str",
                                    "vmSizeProperties": {"vCPUsAvailable": 0, "vCPUsPerCore": 0},
                                },
                                "hyperVGeneration": "str",
                                "licenseType": "str",
                                "location": "str",
                                "osProfile": {
                                    "adminPassword": "str",
                                    "adminUsername": "str",
                                    "allowExtensionOperations": bool,
                                    "computerName": "str",
                                    "customData": "str",
                                    "linuxConfiguration": {
                                        "disablePasswordAuthentication": bool,
                                        "enableVMAgentPlatformUpdates": bool,
                                        "patchSettings": {
                                            "assessmentMode": "str",
                                            "automaticByPlatformSettings": {
                                                "bypassPlatformSafetyChecksOnUserSchedule": bool,
                                                "rebootSetting": "str",
                                            },
                                            "patchMode": "str",
                                        },
                                        "provisionVMAgent": bool,
                                        "ssh": {"publicKeys": [{"keyData": "str", "path": "str"}]},
                                    },
                                    "requireGuestProvisionSignal": bool,
                                    "secrets": [
                                        {
                                            "sourceVault": {"id": "str"},
                                            "vaultCertificates": [{"certificateStore": "str", "certificateUrl": "str"}],
                                        }
                                    ],
                                    "windowsConfiguration": {
                                        "additionalUnattendContent": [
                                            {
                                                "componentName": "str",
                                                "content": "str",
                                                "passName": "str",
                                                "settingName": "str",
                                            }
                                        ],
                                        "enableAutomaticUpdates": bool,
                                        "enableVMAgentPlatformUpdates": bool,
                                        "patchSettings": {
                                            "assessmentMode": "str",
                                            "automaticByPlatformSettings": {
                                                "bypassPlatformSafetyChecksOnUserSchedule": bool,
                                                "rebootSetting": "str",
                                            },
                                            "enableHotpatching": bool,
                                            "patchMode": "str",
                                        },
                                        "provisionVMAgent": bool,
                                        "timeZone": "str",
                                        "winRM": {"listeners": [{"certificateUrl": "str", "protocol": "str"}]},
                                    },
                                },
                                "securityProfile": {
                                    "encryptionAtHost": bool,
                                    "encryptionIdentity": {"userAssignedIdentityResourceId": "str"},
                                    "proxyAgentSettings": {
                                        "addProxyAgentExtension": bool,
                                        "enabled": bool,
                                        "imds": {"inVMAccessControlProfileReferenceId": "str", "mode": "str"},
                                        "keyIncarnationId": 0,
                                        "mode": "str",
                                        "wireServer": {"inVMAccessControlProfileReferenceId": "str", "mode": "str"},
                                    },
                                    "securityType": "str",
                                    "uefiSettings": {"secureBootEnabled": bool, "vTpmEnabled": bool},
                                },
                                "storageProfile": {
                                    "dataDisks": [
                                        {
                                            "caching": "str",
                                            "diskRestorePoint": {
                                                "encryption": {"diskEncryptionSet": {"id": "str"}, "type": "str"},
                                                "id": "str",
                                                "sourceDiskRestorePoint": {"id": "str"},
                                            },
                                            "diskSizeGB": 0,
                                            "lun": 0,
                                            "managedDisk": {
                                                "diskEncryptionSet": {"id": "str"},
                                                "id": "str",
                                                "securityProfile": {
                                                    "diskEncryptionSet": {"id": "str"},
                                                    "securityEncryptionType": "str",
                                                },
                                                "storageAccountType": "str",
                                            },
                                            "name": "str",
                                            "writeAcceleratorEnabled": bool,
                                        }
                                    ],
                                    "diskControllerType": "str",
                                    "osDisk": {
                                        "caching": "str",
                                        "diskRestorePoint": {
                                            "encryption": {"diskEncryptionSet": {"id": "str"}, "type": "str"},
                                            "id": "str",
                                            "sourceDiskRestorePoint": {"id": "str"},
                                        },
                                        "diskSizeGB": 0,
                                        "encryptionSettings": {
                                            "diskEncryptionKey": {"secretUrl": "str", "sourceVault": {"id": "str"}},
                                            "enabled": bool,
                                            "keyEncryptionKey": {"keyUrl": "str", "sourceVault": {"id": "str"}},
                                        },
                                        "managedDisk": {
                                            "diskEncryptionSet": {"id": "str"},
                                            "id": "str",
                                            "securityProfile": {
                                                "diskEncryptionSet": {"id": "str"},
                                                "securityEncryptionType": "str",
                                            },
                                            "storageAccountType": "str",
                                        },
                                        "name": "str",
                                        "osType": "str",
                                        "writeAcceleratorEnabled": bool,
                                    },
                                },
                                "userData": "str",
                                "vmId": "str",
                            },
                            "sourceRestorePoint": {"id": "str"},
                            "timeCreated": "2020-02-20 00:00:00",
                        },
                        "systemData": {
                            "createdAt": "2020-02-20 00:00:00",
                            "createdBy": "str",
                            "createdByType": "str",
                            "lastModifiedAt": "2020-02-20 00:00:00",
                            "lastModifiedBy": "str",
                            "lastModifiedByType": "str",
                        },
                        "type": "str",
                    }
                ],
                "source": {"id": "str", "location": "str"},
            },
            "tags": {"str": "str"},
        },
    )
    print(response)


# x-ms-original-file: 2025-11-01/restorePointExamples/RestorePointCollection_Update_MinimumSet_Gen.json
if __name__ == "__main__":
    main()



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# Below are common methods for the devops build steps. This is the common location that will be updated with
# package targeting during release.

from argparse import Namespace
import sys
import os
from os import path
import re
import logging
from typing import TYPE_CHECKING
from packaging.version import parse

from datetime import date
from ci_tools.functions import discover_targeted_packages
from ci_tools.parsing import ParsedSetup, get_version_py, VERSION_REGEX
from ci_tools.variables import discover_repo_root
from ci_tools.logging import logger

from subprocess import run

VERSION_STRING = 'VERSION = "%s"'

DEV_STATUS_REGEX = r'(classifiers\s*=\s*\[(\s)*)(["\']Development Status :: .*["\'])'

logging.getLogger().setLevel(logging.INFO)

from typing import List


def path_excluded(path, additional_excludes):
    return (
        any([excl in path for excl in additional_excludes])
        or "tests" in os.path.normpath(path).split(os.sep)
        or ParsedSetup.from_path(path).is_metapackage
    )


def get_setup_py_paths(glob_string, base_path, additional_excludes):
    setup_paths = discover_targeted_packages(glob_string, base_path)
    filtered_paths = [p for p in setup_paths if not path_excluded(p, additional_excludes)]
    return filtered_paths


def get_packages(
    args: Namespace, package_name: str = "", additional_excludes: List[str] = [], root_dir: str = None
) -> List[ParsedSetup]:
    if root_dir is None:
        root_dir = discover_repo_root()
    # This function returns list of path to setup.py and setup info like install requires, version for all packages discovered using glob.
    if args.service:
        target_dir = path.join(root_dir, "sdk", args.service)
    else:
        target_dir = root_dir

    paths = get_setup_py_paths(args.glob_string, target_dir, additional_excludes)

    # Check if package is excluded if a package name param is passed
    if package_name and not any(filter(lambda x: package_name == os.path.basename(x), paths)):
        logging.info("Package {} is excluded from version update tool".format(package_name))
        exit(0)

    packages = []
    for setup_path in paths:
        try:
            setup_info = ParsedSetup.from_path(setup_path)
            packages.append(setup_info)
        except:
            print("Error parsing {}".format(setup_path))
            raise

    return packages


def set_version_py(setup_path, new_version):
    version_py_location = get_version_py(setup_path)

    if not version_py_location:
        if "nspkg" in setup_path:
            logger.warning(f"No version.py file found in {setup_path}. This is expected for nspkg packages.")
            return
        logger.error(f"No version.py file found in {setup_path}")
        sys.exit(1)

    version_contents = ""
    with open(version_py_location, "r") as version_py_file:
        version_contents = version_py_file.read()

    with open(version_py_location, "w") as version_py_file:
        replaced_version_contents = re.sub(
            VERSION_REGEX,
            VERSION_STRING % new_version,
            version_contents,
            flags=re.MULTILINE,
        )

        version_py_file.write(replaced_version_contents)


# Get classification for PyPI (https://pypi.org/classifiers/)
def get_classification(version):
    parsed_version = parse(version)
    if not parsed_version.is_prerelease:
        return "Development Status :: 5 - Production/Stable"
    else:
        return "Development Status :: 4 - Beta"


def set_dev_classifier(setup_path, version):
    classification = get_classification(version)

    setup_contents = ""
    with open(setup_path, "r+") as setup_py_file:
        setup_contents = setup_py_file.read()

        # Reset position and truncate the file for new contents
        setup_py_file.seek(0)
        setup_py_file.truncate()

        replaced_setup_contents = re.sub(DEV_STATUS_REGEX, '\g<1>"{}"'.format(classification), setup_contents)

        setup_py_file.write(replaced_setup_contents)


def update_change_log(
    setup_py_location,
    version,
    service,
    package,
    is_unreleased,
    replace_latest_entry_title,
    release_date=None,
    root_dir=None,
):
    if root_dir is None:
        root_dir = discover_repo_root()

    script = os.path.join(root_dir, "eng", "common", "scripts", "Update-ChangeLog.ps1")
    pkg_root = os.path.abspath(os.path.join(setup_py_location, ".."))
    changelog_path = os.path.join(pkg_root, "CHANGELOG.md")
    commands = [
        "pwsh",
        script,
        "--Version",
        version,
        "--ServiceDirectory",
        service,
        "--PackageName",
        package,
        "--Unreleased:${}".format(is_unreleased),
        "--ReplaceLatestEntryTitle:${}".format(replace_latest_entry_title),
        "--ChangelogPath:{}".format(changelog_path),
    ]
    if release_date is not None:
        commands.append("--ReleaseDate:{}".format(release_date))

    # Run script to update change log
    run(commands, cwd=pkg_root)



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.appconfiguration import AppConfigurationManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-appconfiguration
# USAGE
    python configuration_stores_create_with_telemetry.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = AppConfigurationManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.configuration_stores.begin_create(
        resource_group_name="myResourceGroup",
        config_store_name="contoso",
        config_store_creation_parameters={
            "location": "westus",
            "properties": {
                "telemetry": {
                    "resourceId": "/subscriptions/c80fb759-c965-4c6a-9110-9b2b2d038882/resourceGroups/myResourceGroup/providers/microsoft.insights/components/appInsightsName"
                }
            },
            "sku": {"name": "Standard"},
            "tags": {"myTag": "myTagValue"},
        },
    ).result()
    print(response)


# x-ms-original-file: 2025-08-01-preview/ConfigurationStoresCreateWithTelemetry.json
if __name__ == "__main__":
    main()



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.dataprotection import DataProtectionMgmtClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-dataprotection
# USAGE
    python trigger_restore.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = DataProtectionMgmtClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.backup_instances.begin_trigger_restore(
        resource_group_name="000pikumar",
        vault_name="PratikPrivatePreviewVault1",
        backup_instance_name="testInstance1",
        parameters={
            "identityDetails": {
                "useSystemAssignedIdentity": False,
                "userAssignedIdentityArmUrl": "/subscriptions/f75d8d8b-6735-4697-82e1-1a7a3ff0d5d4/resourcegroups/rg-name/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testUami",
            },
            "objectType": "AzureBackupRecoveryPointBasedRestoreRequest",
            "recoveryPointId": "hardcodedRP",
            "restoreTargetInfo": {
                "datasourceAuthCredentials": {
                    "objectType": "SecretStoreBasedAuthCredentials",
                    "secretStoreResource": {
                        "secretStoreType": "AzureKeyVault",
                        "uri": "https://samplevault.vault.azure.net/secrets/credentials",
                    },
                },
                "datasourceInfo": {
                    "datasourceType": "Microsoft.DBforPostgreSQL/servers/databases",
                    "objectType": "Datasource",
                    "resourceID": "/subscriptions/f75d8d8b-6735-4697-82e1-1a7a3ff0d5d4/resourceGroups/viveksipgtest/providers/Microsoft.DBforPostgreSQL/servers/viveksipgtest/databases/targetdb",
                    "resourceLocation": "",
                    "resourceName": "targetdb",
                    "resourceType": "Microsoft.DBforPostgreSQL/servers/databases",
                    "resourceUri": "",
                },
                "datasourceSetInfo": {
                    "datasourceType": "Microsoft.DBforPostgreSQL/servers/databases",
                    "objectType": "DatasourceSet",
                    "resourceID": "/subscriptions/f75d8d8b-6735-4697-82e1-1a7a3ff0d5d4/resourceGroups/viveksipgtest/providers/Microsoft.DBforPostgreSQL/servers/viveksipgtest",
                    "resourceLocation": "",
                    "resourceName": "viveksipgtest",
                    "resourceType": "Microsoft.DBforPostgreSQL/servers",
                    "resourceUri": "",
                },
                "objectType": "RestoreTargetInfo",
                "recoveryOption": "FailIfExists",
                "restoreLocation": "southeastasia",
            },
            "sourceDataStoreType": "VaultStore",
            "sourceResourceId": "/subscriptions/f75d8d8b-6735-4697-82e1-1a7a3ff0d5d4/resourceGroups/viveksipgtest/providers/Microsoft.DBforPostgreSQL/servers/viveksipgtest/databases/testdb",
        },
    ).result()
    print(response)


# x-ms-original-file: 2026-03-01/BackupInstanceOperations/TriggerRestore.json
if __name__ == "__main__":
    main()



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Tests for the candidate resolver module."""

import json
from unittest.mock import patch, MagicMock

import pytest

from azure.ai.agentserver.optimization._resolver import (
    resolve_candidate,
    _downloaded,
    _persist_to_local_layout,
    _download_skill_files,
    _fetch_candidate_config,
    _is_skill_file,
    _build_client,
)
from azure.ai.agentserver.optimization._models import OptimizationConfig


@pytest.fixture(autouse=True)
def clear_downloaded():
    """Clear the downloaded set before each test."""
    _downloaded.clear()
    yield
    _downloaded.clear()


@pytest.fixture()
def mock_client():
    """Return a MagicMock that stands in for PipelineClient."""
    client = MagicMock()
    client._base_url = "http://fake-endpoint"
    return client


ENDPOINT = "http://fake-endpoint"


# ── resolve_candidate ───────────────────────────────────────────────


class TestResolveCandidate:
    """Tests for resolve_candidate function."""

    def test_raises_on_api_failure(self):
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                side_effect=RuntimeError("api failure"),
            ),
        ):
            with pytest.raises(ValueError, match="Failed to fetch config"):
                resolve_candidate("cand-1", endpoint=ENDPOINT)

    def test_returns_config_on_success(self):
        config = {
            "instructions": "Optimized.",
            "model": "gpt-4o",
            "temperature": 0.2,
            "skills": [],
        }
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                return_value=config,
            ),
        ):
            result = resolve_candidate("cand-1", endpoint=ENDPOINT)
            assert result is not None
            assert result["instructions"] == "Optimized."
            assert result["model"] == "gpt-4o"

    def test_uses_correct_path(self):
        """Verify the API route follows /candidates/{candidateId}/config."""
        called_args: list = []

        def capture_call(client, path, params=None):
            called_args.append((path, params))
            return {"instructions": "ok"}

        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                side_effect=capture_call,
            ),
        ):
            resolve_candidate("cand-abc", endpoint="http://api.test")
            assert called_args[0][0] == "/candidates/cand-abc/config"

    def test_marks_downloaded_after_success(self):
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                return_value={"instructions": "ok"},
            ),
        ):
            resolve_candidate("cand-mark", endpoint=ENDPOINT)
            assert "cand-mark" in _downloaded

    def test_skips_if_already_downloaded_and_folder_exists(self, tmp_path):
        """Already-downloaded candidate with existing folder is skipped."""
        (tmp_path / "cand-skip").mkdir()
        _downloaded.add("cand-skip")

        result = resolve_candidate(
            "cand-skip", endpoint=ENDPOINT, local_dir=tmp_path,
        )
        assert result is None

    def test_redownloads_if_folder_missing(self):
        """If downloaded but folder is gone, re-download."""
        _downloaded.add("cand-gone")
        config = {"instructions": "re-downloaded"}
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                return_value=config,
            ),
        ):
            result = resolve_candidate(
                "cand-gone", endpoint=ENDPOINT, local_dir=None,
            )
            # local_dir is None → can't check folder → should re-download
            assert result is not None
            assert result["instructions"] == "re-downloaded"

    def test_does_not_mark_downloaded_on_api_failure(self):
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                side_effect=RuntimeError("api failure"),
            ),
        ):
            with pytest.raises(ValueError):
                resolve_candidate("cand-fail", endpoint=ENDPOINT)
            assert "cand-fail" not in _downloaded


# ── _persist_to_local_layout ────────────────────────────────────────


class TestPersistToLocalLayout:
    """Tests for _persist_to_local_layout."""

    def test_writes_metadata_yaml(self, tmp_path):
        candidate_path = tmp_path / "cand-1"
        config = {"model": "gpt-4o", "temperature": 0.5}
        _persist_to_local_layout(candidate_path, config)

        meta = (candidate_path / "metadata.yaml").read_text()
        assert "model: gpt-4o" in meta
        assert "temperature: 0.5" in meta
        assert f"instruction_file: {OptimizationConfig.INSTRUCTIONS_FILE}" in meta
        assert f"skill_dir: {OptimizationConfig.SKILLS_DIR}" in meta
        assert f"tool_file: {OptimizationConfig.TOOLS_FILE}" in meta

    def test_writes_instructions_md(self, tmp_path):
        candidate_path = tmp_path / "cand-2"
        config = {"instructions": "Hello world."}
        _persist_to_local_layout(candidate_path, config)

        instr = (candidate_path / "instructions.md").read_text()
        assert instr == "Hello world."

    def test_no_instructions_file_when_empty(self, tmp_path):
        candidate_path = tmp_path / "cand-3"
        config = {"model": "gpt-4o"}
        _persist_to_local_layout(candidate_path, config)

        assert not (candidate_path / "instructions.md").exists()

    def test_writes_tools_json_list_format_from_tools_key(self, tmp_path):
        candidate_path = tmp_path / "cand-4"
        config = {
            "tools": [
                {"type": "function", "function": {"name": "search", "description": "Search it", "parameters": {"type": "object", "properties": {"q": {"type": "string", "description": "query"}}}}},
            ]
        }
        _persist_to_local_layout(candidate_path, config)

        tools = json.loads((candidate_path / "tools.json").read_text())
        assert isinstance(tools, list)
        assert tools[0]["function"]["name"] == "search"

    def test_writes_tools_json_multiple_tools(self, tmp_path):
        candidate_path = tmp_path / "cand-5"
        config = {
            "tools": [
                {"type": "function", "function": {"name": "lookup", "description": "Look up policy"}},
                {"type": "function", "function": {"name": "search", "description": "Search"}},
            ]
        }
        _persist_to_local_layout(candidate_path, config)

        tools = json.loads((candidate_path / "tools.json").read_text())
        assert len(tools) == 2
        assert tools[0]["function"]["name"] == "lookup"

    def test_writes_tools_json_list_format(self, tmp_path):
        candidate_path = tmp_path / "cand-6"
        config = {
            "tools": [
                {"type": "function", "function": {"name": "f1", "description": "Func 1"}},
            ]
        }
        _persist_to_local_layout(candidate_path, config)

        tools = json.loads((candidate_path / "tools.json").read_text())
        assert isinstance(tools, list)
        assert tools[0]["function"]["name"] == "f1"

    def test_no_tools_file_when_tools_is_not_list(self, tmp_path):
        """Non-list tools value does not produce tools.json."""
        candidate_path = tmp_path / "cand-7"
        config = {
            "tools": "not a list",
        }
        _persist_to_local_layout(candidate_path, config)

        assert not (candidate_path / "tools.json").exists()

    def test_no_tools_file_when_no_tools(self, tmp_path):
        candidate_path = tmp_path / "cand-8"
        config = {"instructions": "No tools here."}
        _persist_to_local_layout(candidate_path, config)

        assert not (candidate_path / "tools.json").exists()

    def test_overwrites_existing_folder(self, tmp_path):
        candidate_path = tmp_path / "cand-overwrite"
        candidate_path.mkdir()
        (candidate_path / "old_file.txt").write_text("stale")

        config = {"instructions": "Fresh.", "model": "gpt-4o"}
        _persist_to_local_layout(candidate_path, config)

        assert not (candidate_path / "old_file.txt").exists()
        assert (candidate_path / "metadata.yaml").exists()
        assert (candidate_path / "instructions.md").read_text() == "Fresh."

    def test_metadata_without_model_and_temperature(self, tmp_path):
        candidate_path = tmp_path / "cand-minimal"
        config = {"instructions": "Minimal."}
        _persist_to_local_layout(candidate_path, config)

        meta = (candidate_path / "metadata.yaml").read_text()
        assert "model:" not in meta
        assert "temperature:" not in meta

    def test_writes_inline_skills(self, tmp_path):
        """Inline skills are persisted as skills/<name>/SKILL.md with frontmatter."""
        candidate_path = tmp_path / "cand-skills"
        config = {
            "instructions": "With skills.",
            "skills": [
                {"name": "math", "description": "Do math", "body": "Calculate things."},
                {"name": "code", "description": "Write code"},
            ],
        }
        _persist_to_local_layout(candidate_path, config)

        math_skill = candidate_path / "skills" / "math" / "SKILL.md"
        assert math_skill.exists()
        content = math_skill.read_text()
        assert "name: math" in content
        assert "description: Do math" in content
        assert "Calculate things." in content

        code_skill = candidate_path / "skills" / "code" / "SKILL.md"
        assert code_skill.exists()
        code_content = code_skill.read_text()
        assert "name: code" in code_content

    def test_skips_skills_without_name(self, tmp_path):
        """Skills without a name are skipped during persist."""
        candidate_path = tmp_path / "cand-no-name"
        config = {
            "skills": [
                {"description": "no name"},
                {"name": "valid", "description": "has name"},
            ],
        }
        _persist_to_local_layout(candidate_path, config)
        skills_dir = candidate_path / "skills"
        # Only the valid skill should be written
        assert not (skills_dir / "no name").exists()
        assert (skills_dir / "valid" / "SKILL.md").exists()

    def test_skips_non_dict_skills(self, tmp_path):
        """Non-dict skill entries are skipped."""
        candidate_path = tmp_path / "cand-bad-skills"
        config = {"skills": ["not a dict", 42]}
        _persist_to_local_layout(candidate_path, config)
        assert not (candidate_path / "skills").exists()

    def test_skill_frontmatter_uses_yaml_format(self, tmp_path):
        """SKILL.md frontmatter is valid YAML parseable by yaml.safe_load."""
        import yaml

        candidate_path = tmp_path / "cand-yaml-fm"
        config = {
            "skills": [{"name": "search", "description": "Find stuff"}],
        }
        _persist_to_local_layout(candidate_path, config)

        content = (candidate_path / "skills" / "search" / "SKILL.md").read_text()
        assert content.startswith("---\n")
        fm_text = content.split("---")[1]
        parsed = yaml.safe_load(fm_text)
        assert parsed == {"name": "search", "description": "Find stuff"}

    def test_skill_multiline_description_roundtrips(self, tmp_path):
        """Multiline descriptions survive persist → parse round-trip."""
        import yaml

        candidate_path = tmp_path / "cand-multiline"
        desc = "Line one.\nLine two.\nLine three."
        config = {
            "skills": [{"name": "multi", "description": desc, "body": "Body."}],
        }
        _persist_to_local_layout(candidate_path, config)

        content = (candidate_path / "skills" / "multi" / "SKILL.md").read_text()
        fm_text = content.split("---")[1]
        parsed = yaml.safe_load(fm_text)
        assert parsed["description"] == desc
        assert parsed["name"] == "multi"

    def test_skill_description_with_colons(self, tmp_path):
        """Descriptions containing colons are properly quoted in YAML."""
        import yaml

        candidate_path = tmp_path / "cand-colon"
        desc = "USE FOR: search, lookup. DO NOT USE FOR: delete."
        config = {
            "skills": [{"name": "colon", "description": desc}],
        }
        _persist_to_local_layout(candidate_path, config)

        content = (candidate_path / "skills" / "colon" / "SKILL.md").read_text()
        fm_text = content.split("---")[1]
        parsed = yaml.safe_load(fm_text)
        assert parsed["description"] == desc

    def test_skill_description_with_special_chars(self, tmp_path):
        """Descriptions with quotes, brackets, and hashes survive round-trip."""
        import yaml

        candidate_path = tmp_path / "cand-special"
        desc = 'Handle "edge" cases: {x: 1}, [a, b], # comment'
        config = {
            "skills": [{"name": "special", "description": desc}],
        }
        _persist_to_local_layout(candidate_path, config)

        content = (candidate_path / "skills" / "special" / "SKILL.md").read_text()
        fm_text = content.split("---")[1]
        parsed = yaml.safe_load(fm_text)
        assert parsed["description"] == desc

    def test_skill_without_description_has_name_only(self, tmp_path):
        """Skill with no description produces frontmatter with only name."""
        import yaml

        candidate_path = tmp_path / "cand-no-desc"
        config = {
            "skills": [{"name": "bare", "body": "Just body."}],
        }
        _persist_to_local_layout(candidate_path, config)

        content = (candidate_path / "skills" / "bare" / "SKILL.md").read_text()
        fm_text = content.split("---")[1]
        parsed = yaml.safe_load(fm_text)
        assert parsed == {"name": "bare"}
        assert "Just body." in content

    def test_skill_without_body_has_frontmatter_only(self, tmp_path):
        """Skill with no body produces file with frontmatter and no trailing content."""
        candidate_path = tmp_path / "cand-no-body"
        config = {
            "skills": [{"name": "headless", "description": "No body"}],
        }
        _persist_to_local_layout(candidate_path, config)

        content = (candidate_path / "skills" / "headless" / "SKILL.md").read_text()
        # Should be just frontmatter block and trailing newline
        parts = content.strip().split("---")
        # parts[0] is empty (before first ---), parts[1] is frontmatter
        assert len(parts) == 3
        assert parts[2].strip() == ""


# ── _persist + resolve round-trip ────────────────────────────────────


class TestPersistRoundTrip:
    """Ensure persisted layout can be read back by _load_local_dir."""

    def test_round_trip(self, monkeypatch, tmp_path):
        from azure.ai.agentserver.optimization import load_config

        config = {
            "instructions": "Round-trip test.",
            "model": "gpt-4o",
            "temperature": 0.3,
            "tools": [
                {"type": "function", "function": {"name": "search", "description": "Find things", "parameters": {"type": "object", "properties": {"q": {"type": "string", "description": "query"}}}}},
            ],
        }
        candidate_path = tmp_path / "cand-rt"
        _persist_to_local_layout(candidate_path, config)

        # Now load via local dir
        monkeypatch.setenv("OPTIMIZATION_LOCAL_DIR", str(tmp_path))
        monkeypatch.setenv("OPTIMIZATION_CANDIDATE_ID", "cand-rt")
        loaded = load_config()
        assert loaded.instructions == "Round-trip test."
        assert loaded.model == "gpt-4o"
        assert loaded.temperature == 0.3
        assert len(loaded.tool_definitions) == 1
        assert loaded.tool_definitions[0]["function"]["name"] == "search"
        assert loaded.source.startswith("local:")


# ── _download_skill_files ───────────────────────────────────────────


class TestDownloadSkillFiles:
    """Tests for _download_skill_files."""

    def test_downloads_skill_files(self, tmp_path, mock_client):
        candidate_path = tmp_path / "cand-sk"
        candidate_path.mkdir()
        manifest = {
            "files": [
                {"path": "skills/math/SKILL.md", "type": "skill"},
            ]
        }

        def mock_json(client, path, params=None):
            return manifest

        def mock_text(client, path, params=None):
            return "# Math Skill\nDo math."

        with (
            patch("azure.ai.agentserver.optimization._resolver._api_get_json", side_effect=mock_json),
            patch("azure.ai.agentserver.optimization._resolver._api_get_text", side_effect=mock_text),
        ):
            _download_skill_files(mock_client, "cand-sk", candidate_path)

        skill_file = candidate_path / "skills" / "math" / "SKILL.md"
        assert skill_file.exists()
        assert "Math Skill" in skill_file.read_text()

    def test_raises_when_manifest_is_none(self, tmp_path, mock_client):
        candidate_path = tmp_path / "cand-no-manifest"
        candidate_path.mkdir()
        with patch(
            "azure.ai.agentserver.optimization._resolver._api_get_json",
            return_value=None,
        ):
            with pytest.raises(ValueError, match="Invalid manifest"):
                _download_skill_files(mock_client, "cand-no-manifest", candidate_path)

    def test_skips_when_no_skill_files_in_manifest(self, tmp_path, mock_client):
        candidate_path = tmp_path / "cand-no-skills"
        candidate_path.mkdir()
        manifest = {"files": [{"path": "other.txt", "type": "config"}]}
        with patch(
            "azure.ai.agentserver.optimization._resolver._api_get_json",
            return_value=manifest,
        ):
            _download_skill_files(mock_client, "cand-no-skills", candidate_path)
        assert not (candidate_path / "skills").exists()

    def test_raises_on_empty_path_entries(self, tmp_path, mock_client):
        candidate_path = tmp_path / "cand-empty-path"
        candidate_path.mkdir()
        manifest = {"files": [{"path": "", "type": "skill"}]}
        with patch(
            "azure.ai.agentserver.optimization._resolver._api_get_json",
            return_value=manifest,
        ):
            with pytest.raises(ValueError, match="path is empty"):
                _download_skill_files(mock_client, "cand-empty-path", candidate_path)

    def test_raises_on_download_failure(self, tmp_path, mock_client):
        candidate_path = tmp_path / "cand-dl-fail"
        candidate_path.mkdir()
        manifest = {"files": [{"path": "skills/bad/SKILL.md", "type": "skill"}]}
        with (
            patch("azure.ai.agentserver.optimization._resolver._api_get_json", return_value=manifest),
            patch("azure.ai.agentserver.optimization._resolver._api_get_text", return_value=None),
        ):
            with pytest.raises(ValueError, match="Invalid skill file content"):
                _download_skill_files(mock_client, "cand-dl-fail", candidate_path)

    def test_raises_on_download_exception(self, tmp_path, mock_client):
        """Transport error during skill file download raises ValueError."""
        candidate_path = tmp_path / "cand-dl-exc"
        candidate_path.mkdir()
        manifest = {"files": [{"path": "skills/bad/SKILL.md", "type": "skill"}]}
        with (
            patch("azure.ai.agentserver.optimization._resolver._api_get_json", return_value=manifest),
            patch("azure.ai.agentserver.optimization._resolver._api_get_text", side_effect=RuntimeError("timeout")),
        ):
            with pytest.raises(ValueError, match="Failed to download skill file"):
                _download_skill_files(mock_client, "cand-dl-exc", candidate_path)

    def test_rejects_traversal_in_file_path(self, tmp_path, mock_client):
        """File paths with '../' are rejected (zip-slip prevention)."""
        candidate_path = tmp_path / "cand-traversal"
        candidate_path.mkdir()
        manifest = {"files": [{"path": "skills/../../etc/passwd", "type": "skill"}]}
        with (
            patch("azure.ai.agentserver.optimization._resolver._api_get_json", return_value=manifest),
            patch("azure.ai.agentserver.optimization._resolver._api_get_text", return_value="malicious"),
        ):
            with pytest.raises(ValueError, match="Invalid skill file path"):
                _download_skill_files(mock_client, "cand-traversal", candidate_path)
        # Malicious file must NOT be written outside skills_dir
        assert not (tmp_path / "etc" / "passwd").exists()
        assert not (candidate_path / "skills" / ".." / ".." / "etc" / "passwd").exists()

    def test_downloads_multiple_skill_files(self, tmp_path, mock_client):
        """Multiple skill files are downloaded correctly."""
        candidate_path = tmp_path / "cand-multi"
        candidate_path.mkdir()
        manifest = {
            "files": [
                {"path": "skills/math/SKILL.md", "type": "skill"},
                {"path": "skills/code/SKILL.md", "type": "skill"},
            ]
        }
        call_count = {"json": 0, "text": 0}

        def mock_json(client, path, params=None):
            call_count["json"] += 1
            return manifest

        def mock_text(client, path, params=None):
            call_count["text"] += 1
            if "math" in str(params):
                return "# Math\nDo math."
            return "# Code\nWrite code."

        with (
            patch("azure.ai.agentserver.optimization._resolver._api_get_json", side_effect=mock_json),
            patch("azure.ai.agentserver.optimization._resolver._api_get_text", side_effect=mock_text),
        ):
            _download_skill_files(mock_client, "cand-multi", candidate_path)

        assert (candidate_path / "skills" / "math" / "SKILL.md").exists()
        assert (candidate_path / "skills" / "code" / "SKILL.md").exists()

    def test_uses_correct_api_paths(self, tmp_path, mock_client):
        """Verify correct API paths: /candidates/{id} for manifest, /candidates/{id}/files for downloads."""
        candidate_path = tmp_path / "cand-paths"
        candidate_path.mkdir()
        called_paths: list = []

        def mock_json(client, path, params=None):
            called_paths.append(("json", path))
            return {"files": [{"path": "skills/s1/SKILL.md", "type": "skill"}]}

        def mock_text(client, path, params=None):
            called_paths.append(("text", path))
            return "content"

        with (
            patch("azure.ai.agentserver.optimization._resolver._api_get_json", side_effect=mock_json),
            patch("azure.ai.agentserver.optimization._resolver._api_get_text", side_effect=mock_text),
        ):
            _download_skill_files(mock_client, "cand-x", candidate_path)

        assert called_paths[0] == ("json", "/candidates/cand-x")
        assert called_paths[1] == ("text", "/candidates/cand-x/files")

    def test_empty_files_list_in_manifest(self, tmp_path, mock_client):
        """Empty files list in manifest → no downloads."""
        candidate_path = tmp_path / "cand-empty-files"
        candidate_path.mkdir()
        with patch(
            "azure.ai.agentserver.optimization._resolver._api_get_json",
            return_value={"files": []},
        ):
            _download_skill_files(mock_client, "cand-empty-files", candidate_path)
        assert not (candidate_path / "skills").exists()


class TestNormalCandidateId:
    def test_normal_candidate_id_allowed(self, tmp_path):
        """Normal candidate IDs pass the guard."""
        config = {"instructions": "ok", "model": "gpt-4o"}
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                return_value=config,
            ),
        ):
            result = resolve_candidate(
                "valid-candidate-123", endpoint=ENDPOINT, local_dir=tmp_path
            )
            assert result is not None

    def test_candidate_id_with_dots_allowed(self):
        """candidate_id containing dots is fine."""
        config = {"instructions": "ok"}
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                return_value=config,
            ),
        ):
            result = resolve_candidate(
                "candidate.v2.1", endpoint=ENDPOINT
            )
            assert result is not None


# ── resolve_candidate with local_dir ─────────────────────────────────


class TestResolveCandidateWithLocalDir:
    """Tests for resolve_candidate persisting to disk."""

    def test_persists_config_to_disk(self, tmp_path):
        config = {
            "instructions": "Persisted.",
            "model": "gpt-4o",
            "temperature": 0.5,
        }
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                return_value=config,
            ),
        ):
            result = resolve_candidate("cand-persist", endpoint=ENDPOINT, local_dir=tmp_path)
            assert result is not None
            assert (tmp_path / "cand-persist" / "metadata.yaml").exists()
            assert (tmp_path / "cand-persist" / "instructions.md").read_text() == "Persisted."

    def test_sets_skills_dir_when_skills_exist(self, tmp_path):
        """skills_dir is set in returned config when skills folder exists after persist."""
        config = {
            "instructions": "ok",
            "skills": [{"name": "math", "description": "Do math", "body": "# Math"}],
        }
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                return_value=config,
            ),
        ):
            result = resolve_candidate("cand-skills", endpoint=ENDPOINT, local_dir=tmp_path)
            assert result is not None
            assert "skills_dir" in result
            assert "cand-skills" in result["skills_dir"]

    def test_no_skills_dir_when_no_skills(self, tmp_path):
        """skills_dir is not set when no skills are persisted."""
        config = {"instructions": "ok", "model": "gpt-4o"}
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                return_value=config,
            ),
        ):
            result = resolve_candidate("cand-no-skills", endpoint=ENDPOINT, local_dir=tmp_path)
            assert result is not None
            assert "skills_dir" not in result

    def test_local_dir_none_skips_persist(self):
        """When local_dir is None, no files are written."""
        config = {"instructions": "ok"}
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                return_value=config,
            ),
        ):
            result = resolve_candidate("cand-no-dir", endpoint=ENDPOINT, local_dir=None)
            assert result is not None
            assert result["instructions"] == "ok"
            # No skills_dir because no local_dir
            assert "skills_dir" not in result

    def test_endpoint_trailing_slash_stripped(self):
        """Endpoint URL trailing slash doesn't break API paths."""
        called_urls: list = []

        def capture_build(endpoint, credential=None):
            m = MagicMock()
            m._base_url = endpoint
            return m

        def capture_json(client, path, params=None):
            called_urls.append(f"{client._base_url}{path}")
            return {"instructions": "ok"}

        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client", side_effect=capture_build),
            patch("azure.ai.agentserver.optimization._resolver._api_get_json", side_effect=capture_json),
        ):
            resolve_candidate("cand-1", endpoint="http://host/job/123")
            # Verify the URL construction
            assert "/candidates/cand-1/config" in called_urls[0]


# ── _is_skill_file ──────────────────────────────────────────────────


class TestIsSkillFile:
    """Tests for _is_skill_file."""

    def test_type_skill(self):
        assert _is_skill_file({"path": "anything", "type": "skill"})

    def test_path_starts_with_skills(self):
        assert _is_skill_file({"path": "skills/math/SKILL.md", "type": ""})

    def test_not_a_skill(self):
        assert not _is_skill_file({"path": "config.json", "type": "config"})

    def test_empty_entry(self):
        assert not _is_skill_file({})


# ── _fetch_candidate_config ─────────────────────────────────────────


class TestFetchCandidateConfig:
    """Tests for _fetch_candidate_config."""

    def test_wraps_api_error_in_value_error(self, mock_client):
        with patch(
            "azure.ai.agentserver.optimization._resolver._api_get_json",
            side_effect=RuntimeError("connection timeout"),
        ):
            with pytest.raises(ValueError, match="Failed to fetch config"):
                _fetch_candidate_config(mock_client, "cand-err")

    def test_returns_config_on_success(self, mock_client):
        config = {"instructions": "ok", "model": "gpt-4o"}
        with patch(
            "azure.ai.agentserver.optimization._resolver._api_get_json",
            return_value=config,
        ):
            result = _fetch_candidate_config(mock_client, "cand-ok")
            assert result == config


# ── Persist IO error handling ────────────────────────────────────────


class TestPersistErrorHandling:
    """Ensure IO errors during persist don't crash resolve_candidate."""

    def test_persist_oserror_does_not_crash(self, tmp_path):
        config = {"instructions": "ok", "model": "gpt-4o"}
        with (
            patch("azure.ai.agentserver.optimization._resolver._build_client"),
            patch(
                "azure.ai.agentserver.optimization._resolver._api_get_json",
                return_value=config,
            ),
            patch(
                "azure.ai.agentserver.optimization._resolver._persist_to_local_layout",
                side_effect=OSError("disk full"),
            ),
            patch(
                "azure.ai.agentserver.optimization._resolver._download_skill_files",
            ) as mock_download,
        ):
            result = resolve_candidate(
                "cand-io", endpoint=ENDPOINT, local_dir=tmp_path,
            )
            # Config is still returned from API even if persist fails
            assert result is not None
            assert result["instructions"] == "ok"
            assert "cand-io" in _downloaded
            # Skill download is skipped when persist fails
            mock_download.assert_not_called()


# ── HTTP helpers ─────────────────────────────────────────────────────


class TestBuildClient:
    """Tests for _build_client."""

    def test_returns_pipeline_client(self):
        from azure.core import PipelineClient

        client = _build_client("http://example.com")
        assert isinstance(client, PipelineClient)
        client.close()

    def test_works_without_credentials(self):
        """If DefaultAzureCredential fails, client is still created (no auth)."""
        from azure.core import PipelineClient

        with patch(
            "azure.ai.agentserver.optimization._resolver.DefaultAzureCredential",
            side_effect=Exception("No cred"),
        ):
            client = _build_client("http://example.com")
            assert isinstance(client, PipelineClient)
            client.close()



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
DESCRIPTION:
    This sample demonstrates how to perform CRUD operations on Toolboxes
    using the synchronous AIProjectClient.

    It creates two toolbox versions with MCP tools configured with different
    `require_approval` values, switches the default version, and prints the
    MCP `require_approval` setting from the fetched default version.

    Toolboxes are currently a preview feature. In the Python SDK, you access
    these operations via `project_client.beta.toolboxes`.

USAGE:
    python sample_toolboxes_crud.py

    Before running the sample:

    pip install "azure-ai-projects>=2.1.0" python-dotenv

    Set these environment variables with your own values:
    1) FOUNDRY_PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
       page of your Microsoft Foundry portal.
"""

import logging
import os
import sys

from dotenv import load_dotenv

from azure.core.exceptions import ResourceNotFoundError
from azure.identity import DefaultAzureCredential

from azure.ai.projects import AIProjectClient
from azure.ai.projects.models import MCPTool, Tool

load_dotenv()

endpoint = os.environ["FOUNDRY_PROJECT_ENDPOINT"]


def print_mcp_require_approval(tools: list[Tool]) -> None:
    for tool in tools:
        if isinstance(tool, MCPTool):
            print(f"  - MCP `{tool.server_label}` require_approval: {tool.require_approval}")


with (
    DefaultAzureCredential() as credential,
    AIProjectClient(endpoint=endpoint, credential=credential, logging_enable=True) as project_client,
):

    toolbox_name = "toolbox_with_mcp_tool"

    try:
        project_client.beta.toolboxes.delete(toolbox_name)
        print(f"Toolbox `{toolbox_name}` deleted")
    except ResourceNotFoundError:
        pass

    tools_with_mcp_approval_required: list[Tool] = [
        MCPTool(
            server_label="api_specs",
            server_url="https://gitmcp.io/Azure/azure-rest-api-specs",
            require_approval="never",
        )
    ]

    tools_with_mcp_approval_always: list[Tool] = [
        MCPTool(
            server_label="api_specs",
            server_url="https://gitmcp.io/Azure/azure-rest-api-specs",
            require_approval="always",
        )
    ]

    created = project_client.beta.toolboxes.create_version(
        name=toolbox_name,
        description="Toolbox version with MCP require_approval set to 'never'.",
        tools=tools_with_mcp_approval_required,
    )
    print(f"Created toolbox: {created.name} with MCP tools requiring approval 'never' in version {created.version}")

    created = project_client.beta.toolboxes.create_version(
        name=toolbox_name,
        description="Toolbox version with MCP require_approval set to 'always'.",
        tools=tools_with_mcp_approval_always,
    )
    print(f"Created toolbox: {created.name} with MCP tools requiring approval 'always' in version {created.version}")

    updated = project_client.beta.toolboxes.update(
        toolbox_name,
        default_version="2",
    )
    print(f"Updated toolbox: {updated.name} default version is now {updated.default_version}")

    fetched = project_client.beta.toolboxes.get(name=toolbox_name)
    print(f"Retrieved toolbox with default version: {fetched.default_version}")
    fetched_version = project_client.beta.toolboxes.get_version(
        name=toolbox_name,
        version=fetched.default_version,
    )
    print_mcp_require_approval(fetched_version.tools)

    updated = project_client.beta.toolboxes.update(
        toolbox_name,
        default_version="1",
    )
    print(f"Updated toolbox: {updated.name} default version is now {updated.default_version}")

    fetched = project_client.beta.toolboxes.get(name=toolbox_name)
    print(f"Retrieved toolbox with default version: {fetched.default_version}")
    fetched_version = project_client.beta.toolboxes.get_version(
        name=toolbox_name,
        version=fetched.default_version,
    )
    print_mcp_require_approval(fetched_version.tools)

    toolboxes = list(project_client.beta.toolboxes.list())
    print(f"Found {len(toolboxes)} toolboxes")
    for item in toolboxes:
        print(f"  - {item.name} ({item.id})")

    project_client.beta.toolboxes.delete(name=toolbox_name)
    print("Toolbox deleted")



from ci_tools.parsing import parse_require, ParsedSetup
from packaging.specifiers import SpecifierSet
import os
from unittest.mock import patch

import pytest

package_root = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
test_folder = os.path.join(
    os.path.dirname(__file__),
)
scenarios_folder = os.path.join(os.path.dirname(__file__), "integration", "scenarios")
metapackage_scenario = os.path.join(scenarios_folder, "setup_py_metapackage")
setup_project_scenario = os.path.join(scenarios_folder, "setup_py_project_def")
pyproject_scenario = os.path.join(scenarios_folder, "pyproject_project_def")
pyproject_extension_scenario = os.path.join(scenarios_folder, "pyproject_project_def_with_extension")


def test_parse_require():
    test_scenarios = [
        ("ConfigArgParse>=0.12.0", "ConfigArgParse", ">=0.12.0"),
        ("msrest>=0.6.10", "msrest", ">=0.6.10"),
        ("azure-core<2.0.0,>=1.2.2", "azure-core", "<2.0.0,>=1.2.2"),
        ("msrest==0.6.10", "msrest", "==0.6.10"),
        ("msrest<0.6.10", "msrest", "<0.6.10"),
        ("msrest>0.6.9", "msrest", ">0.6.9"),
        ("azure-core<2.0.0,>=1.2.2", "azure-core", "<2.0.0,>=1.2.2"),
        ("azure-core[aio]<2.0.0,>=1.26.0", "azure-core", "<2.0.0,>=1.26.0"),
        ("azure-core[aio,cool_extra]<2.0.0,>=1.26.0", "azure-core", "<2.0.0,>=1.26.0"),
        ("azure-core[]", "azure-core", None),
    ]

    for scenario in test_scenarios:
        result = parse_require(scenario[0])
        assert result.name is not None
        if scenario[2] is not None:
            assert len(result.specifier) != 0
        assert result.name == scenario[1]
        assert str(result.specifier) == (scenario[2] or "")


def test_parse_require_with_no_spec():
    spec_scenarios = ["readme_renderer"]

    for scenario in spec_scenarios:
        result = parse_require(scenario)

        assert result.name == scenario
        assert len(result.specifier) == 0


@patch("ci_tools.parsing.parse_functions.read_setup_py_content")
def test_sdk_sample_setup(test_patch):
    test_patch.return_value = """
import re
import os.path
from io import open
from setuptools import find_packages, setup  # type: ignore

# Change the PACKAGE_NAME only to change folder and different name
PACKAGE_NAME = "azure-core"
PACKAGE_PPRINT_NAME = "Core"

# a-b-c => a/b/c
package_folder_path = PACKAGE_NAME.replace('-', '/')
# a-b-c => a.b.c
namespace_name = PACKAGE_NAME.replace('-', '.')

version = "1.21.0"
readme = "a buncha readme content"
changelog = "some other readme content"

setup(
    name=PACKAGE_NAME,
    version=version,
    include_package_data=True,
    description='Microsoft Azure {} Library for Python'.format(PACKAGE_PPRINT_NAME),
    long_description=readme + changelog,
    long_description_content_type='text/markdown',
    license='MIT License',
    author='Microsoft Corporation',
    author_email='azpysdkhelp@microsoft.com',
    url='https://github.com/Azure/azure-sdk-for-python/tree/main/sdk/core/azure-core',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        'Programming Language :: Python',
        "Programming Language :: Python :: 3 :: Only",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'License :: OSI Approved :: MIT License',
    ],
    zip_safe=False,
    packages=find_packages(exclude=[
        'tests',
        # Exclude packages that will be covered by PEP420 or nspkg
        'azure',
    ]),
    package_data={
        'pytyped': ['py.typed'],
    },
    python_requires=">=3.7",
    keywords=["azure sdk", "hello world"],
    install_requires=[
        'requests>=2.18.4',
        'six>=1.11.0',
        "typing-extensions>=4.0.1",
    ],
)
    """

    result = ParsedSetup.from_path(setup_project_scenario)

    assert result.name == "azure-core"
    assert result.version == "1.21.0"
    assert result.python_requires == ">=3.7"
    assert result.requires == ["requests>=2.18.4", "six>=1.11.0", "typing-extensions>=4.0.1"]
    assert result.is_new_sdk == True
    assert result.setup_filename == os.path.join(setup_project_scenario, "setup.py")
    assert "pytyped" in result.package_data
    assert result.include_package_data == True
    assert result.folder == setup_project_scenario
    assert len(result.classifiers) > 0
    assert result.classifiers[0] == "Development Status :: 5 - Production/Stable"
    assert result.classifiers[5] == "Programming Language :: Python :: 3.8"
    assert result.keywords[0] == "azure sdk"
    assert len(result.keywords) == 2


@patch("ci_tools.parsing.parse_functions.read_setup_py_content")
def test_parse_recognizes_extensions(test_patch):
    test_patch.return_value = """
import re
import os.path
from io import open
import glob
from setuptools import find_packages, setup, Extension  # type: ignore
# Change the PACKAGE_NAME only to change folder and different name
PACKAGE_NAME = "azure-storage-extensions"
PACKAGE_PPRINT_NAME = "Storage Extensions"
# a-b-c => a/b/c
package_folder_path = PACKAGE_NAME.replace('-', '/')
# a-b-c => a.b.c
namespace_name = PACKAGE_NAME.replace('-', '.')
version = "1.21.0"
readme = "a buncha readme content"
changelog = "some other readme content"
setup(
    name=PACKAGE_NAME,
    version=version,
    include_package_data=True,
    description='Microsoft Azure {} Library for Python'.format(PACKAGE_PPRINT_NAME),
    long_description=readme + changelog,
    long_description_content_type='text/markdown',
    license='MIT License',
    author='Microsoft Corporation',
    author_email='azpysdkhelp@microsoft.com',
    url='https://github.com/Azure/azure-sdk-for-python/tree/main/sdk/core/azure-core',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        'Programming Language :: Python',
        "Programming Language :: Python :: 3 :: Only",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'License :: OSI Approved :: MIT License',
    ],
    zip_safe=False,
    packages=find_packages(exclude=[
        'tests',
        # Exclude packages that will be covered by PEP420 or nspkg
        'azure',
    ]),
    ext_package='azure.storage.extensions',
    ext_modules=[
        Extension('crc64', glob.glob(os.path.join(package_folder_path, 'crc64', '*.c')))
    ],
    package_data={
        'pytyped': ['py.typed'],
    },
    python_requires=">=3.7",
    install_requires=[
        'requests>=2.18.4',
        'six>=1.11.0',
        "typing-extensions>=4.0.1",
    ],
)
    """

    result = ParsedSetup.from_path(setup_project_scenario)

    assert result.name == "azure-storage-extensions"
    assert result.version == "1.21.0"
    assert result.python_requires == ">=3.7"
    assert result.requires == ["requests>=2.18.4", "six>=1.11.0", "typing-extensions>=4.0.1"]
    # todo resolve this conflict assert result.is_new_sdk == True
    assert result.setup_filename == os.path.join(setup_project_scenario, "setup.py")
    assert "pytyped" in result.package_data
    assert result.include_package_data == True
    assert result.folder == setup_project_scenario
    assert len(result.classifiers) > 0
    assert result.classifiers[0] == "Development Status :: 5 - Production/Stable"
    assert result.classifiers[5] == "Programming Language :: Python :: 3.8"
    assert result.ext_package == "azure.storage.extensions"
    assert result.ext_modules is not None
    assert result.is_pyproject == False
    assert len(result.ext_modules) == 1
    assert str(type(result.ext_modules[0])) == "<class 'setuptools.extension.Extension'>"


def test_metapackage_detection():
    parsed_project = ParsedSetup.from_path(metapackage_scenario)
    assert parsed_project.is_metapackage == True
    assert parsed_project.name == "azure-keyvault"


def test_parse_pyproject():
    # ensure that we can parse from a folder and a specific file
    parsed_project = ParsedSetup.from_path(pyproject_scenario)

    assert parsed_project.name == "azure-keyvault-keys"
    assert parsed_project.version == "0.0.1"
    assert parsed_project.requires == [
        "azure-common~=1.1",
        "azure-core<2.0.0,>=1.24.0",
        "cryptography>=2.1.4",
        "isodate>=0.6.1",
        "typing-extensions>=4.0.1",
    ]
    assert parsed_project.python_requires == ">=3.8"
    assert parsed_project.is_new_sdk == True
    assert parsed_project.is_pyproject == True
    assert parsed_project.package_data == {"py.typed": ["py.typed"]}
    assert parsed_project.include_package_data == True
    assert parsed_project.folder == pyproject_scenario
    assert parsed_project.is_metapackage == False
    assert parsed_project.namespace == "azure.keyvault.keys"


def test_parse_pyproject_extensions():
    # ensure that we can parse from a folder and a specific file
    parsed_project = ParsedSetup.from_path(pyproject_extension_scenario)

    assert parsed_project.name == "azure-keyvault-keys"
    assert parsed_project.version == "0.0.1b1"
    assert parsed_project.requires == [
        "azure-common~=1.1",
        "azure-core<2.0.0,>=1.24.0",
        "cryptography>=2.1.4",
        "isodate>=0.6.1",
        "typing-extensions>=4.0.1",
    ]
    assert parsed_project.python_requires == ">=3.8"
    assert parsed_project.is_new_sdk == True
    assert parsed_project.is_pyproject == True
    assert parsed_project.package_data == {"py.typed": ["py.typed"]}
    assert parsed_project.include_package_data == True
    assert parsed_project.folder == pyproject_extension_scenario
    assert parsed_project.namespace == "azure.keyvault.keys"
    assert parsed_project.ext_package == "azure.keyvault.keys"
    assert parsed_project.ext_modules is not None
    assert parsed_project.is_metapackage == False
    assert len(parsed_project.ext_modules) == 1
    assert str(type(parsed_project.ext_modules[0])) == "<class 'setuptools.extension.Extension'>"


def test_namespace_discovery_eventhub_checkpointstoreblob():
    """Test that namespace discovery works for azure-eventhub-checkpointstoreblob"""
    eventhub_path = os.path.join(
        os.path.dirname(__file__), "..", "..", "..", "sdk", "eventhub", "azure-eventhub-checkpointstoreblob"
    )

    # Check if the path exists (it should in the Azure SDK repo)
    if os.path.exists(eventhub_path):
        parsed_project = ParsedSetup.from_path(eventhub_path)

        assert parsed_project.name == "azure-eventhub-checkpointstoreblob"
        assert parsed_project.namespace == "azure.eventhub.extensions.checkpointstoreblob"
    else:
        pytest.skip("azure-eventhub-checkpointstoreblob not found in repository")


def test_namespace_discovery_fallback():
    """Test that namespace discovery falls back to simple replacement when no packages found"""
    # This tests the fallback behavior when no actual package structure is found
    from ci_tools.parsing.parse_functions import discover_namespace

    # Test with non-existent path
    result = discover_namespace("/non/existent/path")
    assert result is None


def test_namespace_discovery_with_extension_only():
    """Test namespace discovery logic with extension-only __init__.py files"""
    from ci_tools.parsing.parse_functions import _set_root_namespace
    import tempfile
    import os

    # Create a temporary __init__.py file with only extension content
    with tempfile.NamedTemporaryFile(mode="w", suffix="__init__.py", delete=False) as f:
        f.write("# comment\n")
        f.write('__path__ = __import__("pkgutil").extend_path(__path__, __name__)\n')
        temp_file = f.name

    try:
        result = _set_root_namespace(temp_file, "test.module")
        # Should return None because it only contains extension logic
        assert result is None
    finally:
        os.unlink(temp_file)


def test_namespace_discovery_with_substantial_content():
    """Test namespace discovery logic with substantial __init__.py content"""
    from ci_tools.parsing.parse_functions import _set_root_namespace
    import tempfile
    import os

    # Create a temporary __init__.py file with substantial content
    with tempfile.NamedTemporaryFile(mode="w", suffix="__init__.py", delete=False) as f:
        f.write("# comment\n")
        f.write("from ._version import VERSION\n")
        f.write("__version__ = VERSION\n")
        temp_file = f.name

    try:
        result = _set_root_namespace(temp_file, "test.module")
        # Should return the module name because it contains substantial content
        assert result == "test.module"
    finally:
        os.unlink(temp_file)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.confluent import ConfluentManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestConfluentManagementValidationsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ConfluentManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_validations_validate_organization(self, resource_group):
        response = self.client.validations.validate_organization(
            resource_group_name=resource_group.name,
            organization_name="str",
            body={
                "location": "str",
                "properties": {
                    "offerDetail": {
                        "id": "str",
                        "planId": "str",
                        "planName": "str",
                        "publisherId": "str",
                        "termUnit": "str",
                        "privateOfferId": "str",
                        "privateOfferIds": ["str"],
                        "status": "str",
                        "termId": "str",
                    },
                    "userDetail": {
                        "emailAddress": "str",
                        "aadEmail": "str",
                        "firstName": "str",
                        "lastName": "str",
                        "userPrincipalName": "str",
                    },
                    "createdTime": "2020-02-20 00:00:00",
                    "linkOrganization": {"token": "str"},
                    "organizationId": "str",
                    "provisioningState": "str",
                    "ssoUrl": "str",
                },
                "id": "str",
                "name": "str",
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_validations_validate_organization_v2(self, resource_group):
        response = self.client.validations.validate_organization_v2(
            resource_group_name=resource_group.name,
            organization_name="str",
            body={
                "location": "str",
                "properties": {
                    "offerDetail": {
                        "id": "str",
                        "planId": "str",
                        "planName": "str",
                        "publisherId": "str",
                        "termUnit": "str",
                        "privateOfferId": "str",
                        "privateOfferIds": ["str"],
                        "status": "str",
                        "termId": "str",
                    },
                    "userDetail": {
                        "emailAddress": "str",
                        "aadEmail": "str",
                        "firstName": "str",
                        "lastName": "str",
                        "userPrincipalName": "str",
                    },
                    "createdTime": "2020-02-20 00:00:00",
                    "linkOrganization": {"token": "str"},
                    "organizationId": "str",
                    "provisioningState": "str",
                    "ssoUrl": "str",
                },
                "id": "str",
                "name": "str",
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8
import pytest
from azure.mgmt.databricks.aio import AzureDatabricksManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.live_test_only
class TestAzureDatabricksManagementOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(AzureDatabricksManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_operations_list(self, resource_group):
        response = self.client.operations.list()
        result = [r async for r in response]
        assert len(result)



# pylint: disable=line-too-long,useless-suppression
# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
TEST FILE: test_sample_create_classifier_async.py

DESCRIPTION:
    These tests validate the sample_create_classifier_async.py sample code.
    This sample demonstrates how to create a classifier analyzer to categorize documents and use it
    to analyze documents with and without automatic segmentation.

USAGE:
    pytest test_sample_create_classifier_async.py
"""

import os
import pytest
import uuid
from typing import Dict
from devtools_testutils.aio import recorded_by_proxy_async
from testpreparer_async import ContentUnderstandingPreparer, ContentUnderstandingClientTestBaseAsync
from azure.ai.contentunderstanding import to_llm_input
from azure.ai.contentunderstanding.models import (
    ContentAnalyzer,
    ContentAnalyzerConfig,
    ContentCategoryDefinition,
    DocumentContent,
)


class TestSampleCreateClassifierAsync(ContentUnderstandingClientTestBaseAsync):
    """Tests for sample_create_classifier.py (async version)"""

    @ContentUnderstandingPreparer()
    @recorded_by_proxy_async
    async def test_sample_create_classifier_async(self, contentunderstanding_endpoint: str, **kwargs) -> Dict[str, str]:
        """Test creating a custom classifier with content categories (async version).

        This test validates:
        1. Content categories definition
        2. Analyzer configuration with segmentation
        3. Classifier creation

        05_CreateClassifier.CreateClassifierAsync()
        """
        # Get variables from test proxy (recorded values in playback, empty dict in recording)
        variables = kwargs.pop("variables", {})

        client = self.create_async_client(endpoint=contentunderstanding_endpoint)

        # Generate a unique analyzer ID
        # Use variables from recording if available (playback mode), otherwise generate new one (record mode)
        default_analyzer_id = f"test_classifier_{uuid.uuid4().hex[:16]}"
        analyzer_id = variables.setdefault("createClassifierId", default_analyzer_id)

        print(f"[PASS] Classifier ID generated: {analyzer_id}")

        # Define content categories for classification using ContentCategoryDefinition objects.
        # The Invoice category sets analyzer_id="prebuilt-invoice" so matched segments are
        # routed to the prebuilt invoice analyzer for field extraction (mirrors the sample).
        categories = {
            "Loan_Application": ContentCategoryDefinition(
                description="Documents submitted by individuals or businesses to request funding, typically including personal or business details, financial history, loan amount, purpose, and supporting documentation."
            ),
            "Invoice": ContentCategoryDefinition(
                description="Billing documents issued by sellers or service providers to request payment for goods or services, detailing items, prices, taxes, totals, and payment terms.",
                analyzer_id="prebuilt-invoice",
            ),
            "Bank_Statement": ContentCategoryDefinition(
                description="Official statements issued by banks that summarize account activity over a period, including deposits, withdrawals, fees, and balances."
            ),
        }

        # Assertions for categories
        assert categories is not None, "Categories should not be null"
        assert len(categories) == 3, "Should have 3 categories"
        assert categories["Invoice"].analyzer_id == "prebuilt-invoice", (
            "Invoice category should route segments to prebuilt-invoice for field extraction"
        )
        print(f"[PASS] Content categories defined: {len(categories)} categories (Invoice -> prebuilt-invoice)")

        # Validate each category has description
        for cat_name, cat_def in categories.items():
            assert cat_def.description is not None, f"Category {cat_name} should have description"
            assert cat_def.description.strip(), f"Category {cat_name} description should not be empty"

        print("[PASS] All category definitions validated")

        # Create analyzer configuration using ContentAnalyzerConfig model
        config = ContentAnalyzerConfig(
            return_details=True,
            enable_segment=True,  # Enable automatic segmentation by category
            content_categories=categories,
        )

        # Assertions for config
        assert config is not None, "Config should not be null"
        assert config.enable_segment is True, "Segmentation should be enabled"
        assert config.content_categories is not None, "Config should have content categories"
        assert len(config.content_categories) == 3, "Config should have 3 content categories"
        print("[PASS] Classifier configuration created")

        # Create the classifier analyzer using ContentAnalyzer model
        classifier = ContentAnalyzer(
            base_analyzer_id="prebuilt-document",
            description="Custom classifier for financial document categorization",
            config=config,
            models={"completion": "gpt-4.1"},
        )

        # Assertions for classifier
        assert classifier is not None, "Classifier should not be null"
        assert classifier.base_analyzer_id == "prebuilt-document", "Base analyzer should be prebuilt-document"
        assert classifier.models is not None, "Classifier should have models"
        assert "completion" in classifier.models, "Classifier should have completion model"
        print("[PASS] Classifier definition validated")

        # Create the classifier
        try:
            poller = await client.begin_create_analyzer(analyzer_id=analyzer_id, resource=classifier)

            result = await poller.result()

            # Assertions
            assert poller is not None, "Create classifier operation should not be null"
            assert poller.done(), "Operation should be completed"
            print(f"[PASS] Classifier '{analyzer_id}' created successfully")

            assert result is not None, "Create classifier result should not be null"
            print("[PASS] Create classifier result validated")

            # Cleanup
            try:
                await client.delete_analyzer(analyzer_id=analyzer_id)
                print(f"[PASS] Cleanup: Classifier '{analyzer_id}' deleted")
            except Exception as e:
                print(f"[WARN] Cleanup failed: {str(e)}")
        except Exception as e:
            error_msg = str(e)
            print(f"\n[ERROR] Full error message:\n{error_msg}")
            # Let all exceptions fail - don't skip
            raise

        await client.close()
        print("\n[SUCCESS] All test_sample_create_classifier_async assertions passed")

        # Return variables to be recorded for playback mode
        return variables

    @ContentUnderstandingPreparer()
    @recorded_by_proxy_async
    async def test_sample_analyze_with_classifier_async(
        self, contentunderstanding_endpoint: str, **kwargs
    ) -> Dict[str, str]:
        """Test analyzing a document with a classifier to categorize content into segments (async version).

        This test validates:
        1. Create a classifier with segmentation enabled
        2. Analyze a document with the classifier
        3. Verify segments are returned with category information

        Demonstrates: Analyze documents with segmentation (async)
        """
        # Get variables from test proxy (recorded values in playback, empty dict in recording)
        variables = kwargs.pop("variables", {})

        client = self.create_async_client(endpoint=contentunderstanding_endpoint)

        # Generate a unique analyzer ID
        # Use variables from recording if available (playback mode), otherwise generate new one (record mode)
        default_analyzer_id = f"test_classifier_{uuid.uuid4().hex[:16]}"
        analyzer_id = variables.setdefault("analyzeWithClassifierId", default_analyzer_id)

        print(f"[PASS] Classifier ID generated: {analyzer_id}")

        # Define content categories for classification.
        # Invoice category routes matched segments to prebuilt-invoice for field extraction.
        categories = {
            "Loan_Application": ContentCategoryDefinition(
                description="Documents submitted by individuals or businesses to request funding, typically including personal or business details, financial history, loan amount, purpose, and supporting documentation."
            ),
            "Invoice": ContentCategoryDefinition(
                description="Billing documents issued by sellers or service providers to request payment for goods or services, detailing items, prices, taxes, totals, and payment terms.",
                analyzer_id="prebuilt-invoice",
            ),
            "Bank_Statement": ContentCategoryDefinition(
                description="Official statements issued by banks that summarize account activity over a period, including deposits, withdrawals, fees, and balances."
            ),
        }

        # Create analyzer configuration with segmentation enabled
        config = ContentAnalyzerConfig(
            return_details=True,
            enable_segment=True,  # Enable automatic segmentation by category
            content_categories=categories,
        )

        # Create the classifier analyzer
        classifier = ContentAnalyzer(
            base_analyzer_id="prebuilt-document",
            description="Custom classifier for financial document categorization",
            config=config,
            models={"completion": "gpt-4.1"},
        )

        # Create the classifier
        try:
            poller = await client.begin_create_analyzer(analyzer_id=analyzer_id, resource=classifier)
            result = await poller.result()
            print(f"[PASS] Classifier '{analyzer_id}' created successfully")

            # Analyze a document with the classifier
            current_dir = os.path.dirname(os.path.abspath(__file__))
            test_data_dir = os.path.join(os.path.dirname(current_dir), "test_data")
            file_path = os.path.join(test_data_dir, "mixed_financial_docs.pdf")

            # Check if file exists, fail if missing
            if not os.path.exists(file_path):
                raise FileNotFoundError(f"Test data file not found: {file_path}")

            with open(file_path, "rb") as f:
                file_bytes = f.read()

            # Analyze the document
            analyze_poller = await client.begin_analyze_binary(
                analyzer_id=analyzer_id,
                binary_input=file_bytes,
            )
            analyze_result = await analyze_poller.result()

            # Assertions for analyze result
            assert analyze_result is not None, "Analysis result should not be null"
            print("[PASS] Analysis result received")

            assert analyze_result.contents is not None, "Analysis result contents should not be null"
            assert len(analyze_result.contents) > 0, "Analysis result should have at least one content"
            print(f"[PASS] Analysis result contains {len(analyze_result.contents)} content(s)")

            # Verify document content
            document_content = analyze_result.contents[0]
            assert isinstance(document_content, DocumentContent), "Content should be of type DocumentContent"
            print("[PASS] Content is of type DocumentContent")

            # Verify segments (classification results)
            segments = getattr(document_content, "segments", None)
            if segments and len(segments) > 0:
                print(f"[PASS] Document has {len(segments)} segment(s)")
                for idx, segment in enumerate(segments):
                    category = getattr(segment, "category", None)
                    start_page = getattr(segment, "start_page_number", None)
                    end_page = getattr(segment, "end_page_number", None)
                    segment_id = getattr(segment, "segment_id", None)

                    assert category is not None, f"Segment {idx} should have category"
                    assert start_page is not None, f"Segment {idx} should have start_page_number"
                    assert end_page is not None, f"Segment {idx} should have end_page_number"

                    print(f"  Segment {idx}: Category={category}, Pages {start_page}-{end_page}, ID={segment_id}")
                print("[PASS] All segments have required properties")
            else:
                print("[INFO] No segments found (document classified as single unit)")

            # Test to_llm_input conversion (mirrors sample's [START classifier_to_llm_input] block).
            # For classification results, to_llm_input expands the parent into per-segment
            # blocks separated by '*****', each tagged with its category in the YAML front matter.
            text = to_llm_input(analyze_result)
            assert isinstance(text, str) and text.strip(), "to_llm_input should return a non-empty string"
            assert text.startswith("---"), "to_llm_input output should start with YAML front matter delimiter"
            assert "contentType: document" in text, "YAML front matter should declare 'contentType: document'"
            segments = getattr(document_content, "segments", None)
            if segments and len(segments) > 1:
                # Multi-segment classification: per-segment blocks separated by '*****'
                assert "*****" in text, "Multi-segment classification output should include '*****' segment divider"
                # Each segment should appear with its category label in front matter
                for segment in segments:
                    cat = getattr(segment, "category", None)
                    if cat:
                        assert f"category: {cat}" in text, (
                            f"Segment category '{cat}' should appear in YAML front matter"
                        )
            print(f"[PASS] to_llm_input output validated ({len(text)} characters)")

            # Cleanup
            try:
                await client.delete_analyzer(analyzer_id=analyzer_id)
                print(f"[PASS] Cleanup: Classifier '{analyzer_id}' deleted")
            except Exception as e:
                print(f"[WARN] Cleanup failed: {str(e)}")

        except Exception as e:
            error_msg = str(e)
            print(f"\n[ERROR] Full error message:\n{error_msg}")
            # Let all exceptions fail - don't skip
            raise

        await client.close()
        print("\n[SUCCESS] All test_sample_analyze_with_classifier_async assertions passed")

        # Return variables to be recorded for playback mode
        return variables



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.batch import BatchManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-batch
# USAGE
    python batch_account_create_byos.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = BatchManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.batch_account.begin_create(
        resource_group_name="default-azurebatch-japaneast",
        account_name="sampleacct",
        parameters={
            "location": "japaneast",
            "properties": {
                "autoStorage": {
                    "storageAccountId": "/subscriptions/12345678-1234-1234-1234-123456789012/resourceGroups/default-azurebatch-japaneast/providers/Microsoft.Storage/storageAccounts/samplestorage"
                },
                "keyVaultReference": {
                    "id": "/subscriptions/12345678-1234-1234-1234-123456789012/resourceGroups/default-azurebatch-japaneast/providers/Microsoft.KeyVault/vaults/sample",
                    "url": "http://sample.vault.azure.net/",
                },
                "poolAllocationMode": "UserSubscription",
            },
        },
    ).result()
    print(response)


# x-ms-original-file: 2025-06-01/BatchAccountCreate_BYOS.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.applicationinsights import ApplicationInsightsManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestApplicationInsightsManagementComponentsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ApplicationInsightsManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_components_get(self, resource_group):
        response = self.client.components.get(
            resource_group_name=resource_group.name,
            resource_name="str",
            api_version="2020-02-02",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_components_create_or_update(self, resource_group):
        response = self.client.components.create_or_update(
            resource_group_name=resource_group.name,
            resource_name="str",
            insight_properties={
                "kind": "str",
                "location": "str",
                "etag": "str",
                "id": "str",
                "name": "str",
                "properties": {
                    "Application_Type": "str",
                    "AppId": "str",
                    "ApplicationId": "str",
                    "ConnectionString": "str",
                    "CreationDate": "2020-02-20 00:00:00",
                    "DisableIpMasking": bool,
                    "DisableLocalAuth": bool,
                    "Flow_Type": "str",
                    "ForceCustomerStorageForProfiler": bool,
                    "HockeyAppId": "str",
                    "HockeyAppToken": "str",
                    "ImmediatePurgeDataOn30Days": bool,
                    "IngestionMode": "str",
                    "InstrumentationKey": "str",
                    "LaMigrationDate": "2020-02-20 00:00:00",
                    "Name": "str",
                    "PrivateLinkScopedResources": [{"ResourceId": "str", "ScopeId": "str"}],
                    "Request_Source": "str",
                    "RetentionInDays": 0,
                    "SamplingPercentage": 0.0,
                    "TenantId": "str",
                    "WorkspaceResourceId": "str",
                    "provisioningState": "str",
                    "publicNetworkAccessForIngestion": "str",
                    "publicNetworkAccessForQuery": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
            api_version="2020-02-02",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_components_update_tags(self, resource_group):
        response = self.client.components.update_tags(
            resource_group_name=resource_group.name,
            resource_name="str",
            component_tags={"tags": {"str": "str"}},
            api_version="2020-02-02",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_components_delete(self, resource_group):
        response = self.client.components.delete(
            resource_group_name=resource_group.name,
            resource_name="str",
            api_version="2020-02-02",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_components_list_by_resource_group(self, resource_group):
        response = self.client.components.list_by_resource_group(
            resource_group_name=resource_group.name,
            api_version="2020-02-02",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_components_list(self, resource_group):
        response = self.client.components.list(
            api_version="2020-02-02",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_components_purge(self, resource_group):
        response = self.client.components.purge(
            resource_group_name=resource_group.name,
            resource_name="str",
            body={"filters": [{"column": "str", "key": "str", "operator": "str", "value": {}}], "table": "str"},
            api_version="2020-02-02",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_components_get_purge_status(self, resource_group):
        response = self.client.components.get_purge_status(
            resource_group_name=resource_group.name,
            resource_name="str",
            purge_id="str",
            api_version="2020-02-02",
        )

        # please add some check logic here by yourself
        # ...



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.appcontainers import ContainerAppsAPIClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-appcontainers
# USAGE
    python job_createor_update_connected_environment.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ContainerAppsAPIClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.jobs.begin_create_or_update(
        resource_group_name="rg",
        job_name="testcontainerAppsJob0",
        job_envelope={
            "location": "East US",
            "properties": {
                "configuration": {
                    "manualTriggerConfig": {"parallelism": 4, "replicaCompletionCount": 1},
                    "replicaRetryLimit": 10,
                    "replicaTimeout": 10,
                    "triggerType": "Manual",
                },
                "environmentId": "/subscriptions/34adfa4f-cedf-4dc0-ba29-b6d1a69ab345/resourceGroups/rg/providers/Microsoft.App/connectedEnvironments/demokube",
                "template": {
                    "containers": [
                        {
                            "image": "repo/testcontainerAppsJob0:v1",
                            "name": "testcontainerAppsJob0",
                            "probes": [
                                {
                                    "httpGet": {
                                        "httpHeaders": [{"name": "Custom-Header", "value": "Awesome"}],
                                        "path": "/health",
                                        "port": 8080,
                                    },
                                    "initialDelaySeconds": 5,
                                    "periodSeconds": 3,
                                    "type": "Liveness",
                                }
                            ],
                        }
                    ],
                    "initContainers": [
                        {
                            "args": ["-c", "while true; do echo hello; sleep 10;done"],
                            "command": ["/bin/sh"],
                            "image": "repo/testcontainerAppsJob0:v4",
                            "name": "testinitcontainerAppsJob0",
                            "resources": {"cpu": 0.2, "memory": "100Mi"},
                        }
                    ],
                },
            },
        },
    ).result()
    print(response)


# x-ms-original-file: 2026-01-01/Job_CreateorUpdate_ConnectedEnvironment.json
if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.compute import ComputeManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-compute
# USAGE
    python gallery_add_to_sharing_profile.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ComputeManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="SUBSCRIPTION_ID",
    )

    response = client.gallery_sharing_profile.begin_update(
        resource_group_name="myResourceGroup",
        gallery_name="myGalleryName",
        sharing_update={
            "groups": [
                {
                    "ids": ["34a4ab42-0d72-47d9-bd1a-aed207386dac", "380fd389-260b-41aa-bad9-0a83108c370b"],
                    "type": "Subscriptions",
                },
                {"ids": ["c24c76aa-8897-4027-9b03-8f7928b54ff6"], "type": "AADTenants"},
            ],
            "operationType": "Add",
        },
    ).result()
    print(response)


# x-ms-original-file: 2025-03-03/galleryExamples/Gallery_AddToSharingProfile.json
if __name__ == "__main__":
    main()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_swap_deployments.py
DESCRIPTION:
    This sample demonstrates how to swap two **Text Authoring** deployments within a project.
USAGE:
    python sample_swap_deployments.py
REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_TEXT_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET
NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_TEXT_ENDPOINT
      - AZURE_TEXT_KEY
OPTIONAL ENV VARS:
    PROJECT_NAME            # defaults to "<project-name>"
    FIRST_DEPLOYMENT_NAME   # defaults to "<deployment-name-1>"
    SECOND_DEPLOYMENT_NAME  # defaults to "<deployment-name-2>"
"""

# [START text_authoring_swap_deployments]
import os
from azure.identity import DefaultAzureCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.textanalytics.authoring import TextAuthoringClient
from azure.ai.textanalytics.authoring.models import SwapDeploymentsDetails


def sample_swap_deployments():
    # settings
    endpoint = os.environ["AZURE_TEXT_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    first_deployment = os.environ.get("FIRST_DEPLOYMENT_NAME", "<deployment-name-1>")
    second_deployment = os.environ.get("SECOND_DEPLOYMENT_NAME", "<deployment-name-2>")

    # create a client with AAD
    credential = DefaultAzureCredential()
    client = TextAuthoringClient(endpoint, credential=credential)

    # project-scoped client
    project_client = client.get_project_client(project_name)

    # prepare swap details
    details = SwapDeploymentsDetails(
        first_deployment_name=first_deployment,
        second_deployment_name=second_deployment,
    )

    # begin swap (LRO) and handle success/error
    poller = project_client.project.begin_swap_deployments(body=details)
    try:
        poller.result()  # completes with None; raises on failure
        print("Swap deployments completed.")
        print(f"done: {poller.done()}")
        print(f"status: {poller.status()}")
    except HttpResponseError as e:
        print(f"Operation failed: {e.message}")
        print(e.error)


# [END text_authoring_swap_deployments]


def main():
    sample_swap_deployments()


if __name__ == "__main__":
    main()
